/**************************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 				  *
 *  ZL27ARM: PROJEKT_13 Park Dostance Control  																			*
 * 										 licznik TIM4: PWM do silnikow					*
 *										 licznik TIM3: Radio Control						*
 *										 licznik TIM1: pomiar odleglosci 				*
 *																	 kanal4: sygnal Trig 			*
 *																	 kanaly 1 i 2: pomiar			*
 *										 licznik TIM2: kanal 4: brzeczyk				*
 **************************************************************/
#include "stm32f10x.h"
#include <stdbool.h> 		// true, false
#include <stdio.h>
#include ".\..\Drivers\KAmodKB4x4\KB_4x4.h"
#include ".\..\Drivers\LCD1602\lcd_1602.h"

// zmienne globalne do zapisywania znakow z klawiatury ----------
unsigned char key_buffer[32] = {0}; 		// deklaracja tablicy
volatile unsigned int key_pointer = 0;	// numer elementu w tablicy 'key_buffer'
unsigned char txt2LCD[32] = {0}; 				// bufor funkcji sprintf
unsigned char KB2mem[32] = {0}; 				// bufor funkcji sscanf
unsigned int RTC_run = 0;								// zezwala na wyswietlanie czasu
int RC1stick = 0, RC3stick = 0;					// polozenia drazkow 1 i 3 w nadajniku RC
unsigned int IC1, IC2, IC3, IC4;				// odczyty rejestrow Capture z TIM3
int HC_SR04us;													// dlugosc impulsu [us]
unsigned int IC1_1, IC1_2;							// odczyty rejestrow Capture z TIM1

int  RCC_Config(void);
void GPIO_Config(void);
void NVIC_Config(void);
int  SysTick_Config_Div8(unsigned int Ticks);
void RTC_Config(void);
void TIM_Config(void);

int main(void) {
	//------------------- deklaracje zmiennych ------------------
	int MotorL = 0, MotorR = 0;
	unsigned int PWM_R = 520, PWM_R_old = 520, PWM_L = 520, PWM_L_old = 520; 
	float Lcm = 0;
	uint16_t freq = 0;
	
  //------------------ konfiguracja systemu -------------------
  RCC_Config();   
  GPIO_Config(); 
  NVIC_Config();
	RTC_Config();
  GPIO_ResetBits(GPIOB, 0xFF00u);	// zgasimy wszystkie diody LED
	LCD_Initialize();              	//inicjalizacja wyswietlacza
	LCD_SendCommand(LCD_DISPLAY_ONOFF | LCD_DISPLAY_ON | LCD_CURSOR_ON);	
	SysTick_Config_Div8(9000000ul);
	TIM_Config();
	RTC_SetCounter(0);
	
	//------------------------ program C ------------------------
	LCD_WriteTextXY((unsigned char *)"                \0",0,1);
	// petla nieskonczona
	while (1) {
		MotorL = RC1stick + RC3stick;
		if(MotorL > 1043)
			MotorL = 1043;
		else if(MotorL < -1043)
			MotorL = -1043;
		MotorR = RC3stick - RC1stick;
		if(MotorR > 1043)
			MotorR = 1043;
		else if(MotorR < -1043)
			MotorR = -1043;
		// Motor Prawy
		PWM_R = MotorR > 0 ? MotorR : -MotorR; 	// wartosc bezwzgledna
		if(PWM_R != PWM_R_old){
			PWM_R_old = PWM_R;
			TIM_SetCompare3(TIM4, PWM_R);
			if(MotorR > 0){
				GPIO_ResetBits(GPIOB, GPIO_Pin_11);
				GPIO_SetBits(GPIOB, GPIO_Pin_10);
			}
			else if(MotorR < 0){
				GPIO_ResetBits(GPIOB, GPIO_Pin_10);
				GPIO_SetBits(GPIOB, GPIO_Pin_11);
			}
		}
		// Motor Lewy
		PWM_L = MotorL > 0 ? MotorL : -MotorL; 	// wartosc bezwzgledna
		if(PWM_L != PWM_L_old){
			PWM_L_old = PWM_L;
			TIM_SetCompare4(TIM4, PWM_L);
			if(MotorL > 0){
				GPIO_ResetBits(GPIOB, GPIO_Pin_13);
				GPIO_SetBits(GPIOB, GPIO_Pin_12);
			}
			else if(MotorL < 0){
				GPIO_ResetBits(GPIOB, GPIO_Pin_12);
				GPIO_SetBits(GPIOB, GPIO_Pin_13);
			}
		}

		// wyswietlenie zmierzonej odleglosci do przeszkody
		LCD_GoTo(0,0);
		Lcm = HC_SR04us / 58.4371;
		sprintf((char *)txt2LCD,"L = %7.2f [cm]",Lcm);
		LCD_WriteText(txt2LCD);
		// wyswietlenie sterowan
		LCD_GoTo(0,1);
		sprintf((char *)txt2LCD,"FB%5i",RC3stick);
		LCD_WriteText(txt2LCD);    
		LCD_GoTo(9,1);
		sprintf((char *)txt2LCD,"LR%5i",RC1stick);
		LCD_WriteText(txt2LCD);
		
		// dzwiek: brzeczyk
		if(Lcm < 60){
			freq = 100 * Lcm;
			TIM_SetAutoreload(TIM2, freq);
			TIM_Cmd(TIM2, ENABLE);	// Wlaczenie timera TIM2
		}
		else
			TIM_Cmd(TIM2, DISABLE);	// Wylaczenie timera TIM2

		delay_us(800000);
	}
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii 
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); // wlacz taktowanie portu AFIO (przerwania zewnetrzne)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);// wlacz taktowanie portu GPIO A (przyciski i wejscia TIM2)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);// wlacz taktowanie portu GPIO B (LEDy i wyjscia TIM4)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);// wlacz taktowanie portu GPIO C (LCD1602)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);// wlacz taktowanie portu GPIO D (klawiatura)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE); // wlacz taktowanie TIM1 z APB2
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE); // wlacz taktowanie TIM2 z APB1
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE); // wlacz taktowanie TIM3 z APB1
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE); // wlacz taktowanie TIM4 z APB1
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 
	
	// port A - Przyciski, joystick oraz PA6/PA7 = RC receiver, PA8/PA9 = Distance Sensor
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	// port A - PA11 = Distance Sensor
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;      // Alternate function push-pull
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	// port A - PA3 = buzzer
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;      // Alternate function - otwarty kolektor
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	// port B - diody LED i wyjscia TIM4
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	// port B - wyjscia TIM4 - PWM do silnikow
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;      // Alternate function push-pull
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	// port B - PB0/PB1 = RC receiver
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	// port D - klawiatura
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;      // wyjscie w trybie Open-Drain
  GPIO_Init(GPIOD, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;	// wejscie w trybie FLOATING,
  GPIO_Init(GPIOD, &GPIO_InitStructure);
	GPIO_ResetBits(GPIOD, GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13);
	GPIO_AFIODeInit();
}

// ----------- konfiguracja kontrolera przerwan NVIC ----------
void NVIC_Config(void) {
	NVIC_InitTypeDef  NVIC_InitStructure; 
  EXTI_InitTypeDef  EXTI_InitStructure; 
	
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);	// tablica wektorow przerwan we FLASH
	// Konfiguracja NVIC - ustawienia priorytetow przerwania EXTI0
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);                //Wybor modelu grupowania przerwan
	
  // linia 6 = COL1 -------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource6);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 6
  EXTI_InitStructure.EXTI_Line = EXTI_Line6;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	// linia 7 = COL2 -------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource7);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 7
  EXTI_InitStructure.EXTI_Line = EXTI_Line7;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	// linia 8 = COL3 ---------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource8);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 8
  EXTI_InitStructure.EXTI_Line = EXTI_Line8;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	// linia 9 = COL4 ---------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource9);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 9
  EXTI_InitStructure.EXTI_Line = EXTI_Line9;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	
	// Wybor konfigurowanego IRQ ----------------------------------------------------------------
	NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;             //Wybor konfigurowanego IRQ
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;      //Priorytet grupowy
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;             //Podpriorytet
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;                //Wlaczenie obslugi IRQ
  NVIC_Init(&NVIC_InitStructure);
	
	// Wlacz przerwanie od RTC ------------------------------------------------------------------
  NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
	
	// Wlacz przerwanie od TIM1 -----------------------------------------------------------------
  NVIC_InitStructure.NVIC_IRQChannel = TIM1_CC_IRQn;				// TIM1 Capture Compare Interrupt
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
	
	// Wlacz przerwanie od TIM3 -----------------------------------------------------------------
  NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;              	// TIM3 global Interrupt
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
	
}
// --------- konfiguracja licznika systemowego SysTick --------
int SysTick_Config_Div8(unsigned int Ticks){ 
  unsigned int ctrl;

  if (Ticks > SysTick_LOAD_RELOAD_Msk)  return 1;             	// czy wartosc poczatkowa nie przekracza max
                                                                
  SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        // ustaw wartosc poczatkowa licznika 
  NVIC_SetPriority (SysTick_IRQn, 0);                           // ustaw priorytet przerwania
  SysTick->VAL  = 0;                                            // ustaw wartosc aktualna licznika
  ctrl=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;      // ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	ctrl &= SysTick_CLKSource_HCLK_Div8;
  SysTick->CTRL = ctrl;                  // zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)
  return 0;
}
 // -------------------  konfigurowanie RTC -------------------
void RTC_Config(void) {
	RCC -> APB1ENR |= RCC_APB1Periph_PWR;	// Power  interface clock enable
	RCC -> APB1ENR |= RCC_APB1Periph_BKP; // Backup interface clock enable

	PWR -> CR |= PWR_CR_DBP;				// Disable backup domain write protection
	// Select LSE clock
	RCC -> CSR &= ~RCC_CSR_LSION; 	// LSI (w RCC) off
	RCC -> BDCR &= ~RCC_BDCR_LSEBYP;// External Low Speed oscillator Bypass = off
	RCC -> BDCR |= RCC_BDCR_RTCSEL_LSE;	/*!< LSE oscillator clock used as RTC clock */
	RCC -> BDCR |= RCC_BDCR_LSEON;			/*!< External Low Speed oscillator enable */
	while((RCC -> BDCR & RCC_BDCR_LSERDY) == 0)
		;																	// czekamy na gotowosc LSE
	RCC -> BDCR |= RCC_BDCR_RTCEN;			/*!< RTC clock enable */
	
  RTC_WaitForSynchro();                    //Oczekiwanie na synchronizacje
  RTC_WaitForLastTask();                   //Oczekiwanie az rejestry RTC zostana zapisane
  RTC_SetPrescaler(32767);                 //Okres RTC bedzie wynosil 1s  RTC period = RTCCLK/RTC_PR = (32.768 KHz)/(32767+1)
  RTC_WaitForLastTask();
  RTC_ITConfig(RTC_IT_SEC, ENABLE);        // Wlacz przerwanie "sekundnika" RTC 
  RTC_WaitForLastTask();
  RCC_ClearFlag();
}
 // --------------  konfigurowanie licznikow TIMx --------------
void TIM_Config(void) {
  //Konfiguracja timerow
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;
	TIM_ICInitTypeDef	 TIM_ICInitStructure;
	TIM_BDTRInitTypeDef TIM_BDTRInitStruct;
	
  //Konfiguracja licznika 4: sygnaly PWM do silnikow
	TIM_InternalClockConfig(TIM4);	// zeruje bity SMS w rejestrze TIM4_SMCR, w polaczeniu z CEN(TIM4_CR1)=1 daje CK_INT
	//TIM_TimeBaseStructure.TIM_Prescaler = 4-1; 		              //taktowanie licznka fclk = 72MHz/4 = 18 MHz 	VNH2SP30
	TIM_TimeBaseStructure.TIM_Prescaler = 7-1; 		              //taktowanie licznka fclk = 72MHz/7 						VNH3SP30
	TIM_TimeBaseStructure.TIM_Period = 1043-1;                  //okres przepelnien licznika = 1043 takty
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    	//dzielnik zegara dla ukladu generacji dead-time i filtra
 	TIM_TimeBaseStructure.TIM_RepetitionCounter=0;          		//licznik powtorzen
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;	//tryb pracy licznika
	TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);            	//Inicjalizacja licznika
	//Konfiguracja kanalu 3
	TIM_OCInitStructure.TIM_OCMode =  TIM_OCMode_PWM1;         	//tryb pracy kanalu
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;//wlaczenie generowania sygnalu na wyjsciu licznika
	TIM_OCInitStructure.TIM_Pulse = 520;                        // wypelnienie 50%
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;   //polaryzacja wyjscia
	TIM_OC3Init(TIM4, &TIM_OCInitStructure);                    //Inicjalizacja kanalu 3 licznika TIM4	
	TIM_OC3PreloadConfig(TIM4, TIM_OCPreload_Enable);           //zapis ustawien, wlaczenie buforowania
	//Konfiguracja kanalu 4
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;         	//tryb pracy kanalu
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;//wlaczenie generowania sygnalu na wyjsciu licznika
	TIM_OCInitStructure.TIM_Pulse = 520;                        // wypelnienie 50%
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;   //polaryzacja wyjscia
	TIM_OC4Init(TIM4, &TIM_OCInitStructure);                    //Inicjalizacja kanalu 4 licznika TIM4	
	TIM_OC4PreloadConfig(TIM4, TIM_OCPreload_Enable);           //zapis ustawien, wlaczenie buforowania
  TIM_Cmd(TIM4, ENABLE);																			// Wlaczenie timera TIM4
	
	//Konfiguracja licznika 3: pomiar dwoch kanalow z odbiornika RC
	TIM_InternalClockConfig(TIM3);	// zeruje bity SMS w rejestrze TIM3_SMCR, w polaczeniu z CEN(TIM3_CR1)=1 daje CK_INT
	TIM_TimeBaseStructure.TIM_Prescaler = 29-1;              		//taktowanie licznika fclk = 72MHz/29
	TIM_TimeBaseStructure.TIM_Period = 0xFFFF;               	  //okres przepelnien licznika = MAX
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    	//dzielnik zegara dla ukladu generacji dead-time i filtra
 	TIM_TimeBaseStructure.TIM_RepetitionCounter=0;          		//licznik powtorzen
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;	//tryb pracy licznika
	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);            	//Inicjalizacja licznika
	//Konfiguracja kanalu 1
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;         		// numer kanalu
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;  // wejscia dolaczone do ...  TIM_ICSelection_DirectTI
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;       // Capture performed each time an edge is detected on the capture input.
	TIM_ICInitStructure.TIM_ICFilter = 0;												// This parameter can be a number between 0x0 and 0xF */
	TIM_ICInit(TIM3, &TIM_ICInitStructure);
	//Konfiguracja kanalu 2
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;         		// numer kanalu
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Falling;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;  // wejscia dolaczone do ...  TIM_ICSelection_DirectTI
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;       // Capture performed each time an edge is detected on the capture input.
	TIM_ICInitStructure.TIM_ICFilter = 0;												// This parameter can be a number between 0x0 and 0xF */
	TIM_ICInit(TIM3, &TIM_ICInitStructure);
	//Konfiguracja kanalu 3
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_3;         		// numer kanalu
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;  // wejscia dolaczone do ...  TIM_ICSelection_DirectTI
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;       // Capture performed each time an edge is detected on the capture input.
	TIM_ICInitStructure.TIM_ICFilter = 0;												// This parameter can be a number between 0x0 and 0xF */
	TIM_ICInit(TIM3, &TIM_ICInitStructure);
	//Konfiguracja kanalu 4
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_4;         		// numer kanalu
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Falling;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;  // wejscia dolaczone do ...  TIM_ICSelection_DirectTI
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;       // Capture performed each time an edge is detected on the capture input.
	TIM_ICInitStructure.TIM_ICFilter = 0;												// This parameter can be a number between 0x0 and 0xF */
	TIM_ICInit(TIM3, &TIM_ICInitStructure);

	TIM_Cmd(TIM3, ENABLE);																			// Wlaczenie timera TIM3
  TIM_ITConfig(TIM3, TIM_IT_CC1, ENABLE);  //wlaczenie przerwania od CC1 dla TIM3
	TIM_ITConfig(TIM3, TIM_IT_CC2, ENABLE);  //wlaczenie przerwania od CC2 dla TIM3
	TIM_ITConfig(TIM3, TIM_IT_CC3, ENABLE);  //wlaczenie przerwania od CC3 dla TIM3
	TIM_ITConfig(TIM3, TIM_IT_CC4, ENABLE);  //wlaczenie przerwania od CC4 dla TIM3
	
	//Konfiguracja licznika 2: dzwiek: przetwornik piezoelektryczny
	TIM_InternalClockConfig(TIM2);	// zeruje bity SMS w rejestrze TIM2_SMCR, w polaczeniu z CEN(TIM2_CR1)=1 daje CK_INT
	TIM_TimeBaseStructure.TIM_Prescaler = 72-1; 		            //taktowanie licznka fclk = 72MHz/72 = 1 MHz
	TIM_TimeBaseStructure.TIM_Period = 5000-1;                  //okres przepelnien licznika = 1
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    	//dzielnik zegara dla ukladu generacji dead-time i filtra
 	TIM_TimeBaseStructure.TIM_RepetitionCounter=0;          		//licznik powtorzen
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;	//tryb pracy licznika
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);            	//Inicjalizacja licznika
	//Konfiguracja kanalu 4
	TIM_OCInitStructure.TIM_OCMode =  TIM_OCMode_Toggle;        //tryb pracy kanalu
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;//wlaczenie generowania sygnalu na wyjsciu licznika
	TIM_OCInitStructure.TIM_Pulse = 20;                         // przesuniecie
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;   //polaryzacja wyjscia
	TIM_OC4Init(TIM2, &TIM_OCInitStructure);                    //Inicjalizacja kanalu 4 licznika TIM2	
	TIM_OC4PreloadConfig(TIM2, TIM_OCPreload_Enable);           //zapis ustawien, wlaczenie buforowania
	
	//Konfiguracja licznika 1: pomiar odleglosci
	TIM_InternalClockConfig(TIM1);	// zeruje bity SMS w rejestrze TIM1_SMCR, w polaczeniu z CEN(TIM1_CR1)=1 daje CK_INT
	TIM_TimeBaseStructure.TIM_Prescaler = 72-1; 		            //taktowanie licznka fclk = 72MHz/72 = 1 MHz
	TIM_TimeBaseStructure.TIM_Period = 0xFFFF;                  //okres przepelnien licznika = MAX
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    	//dzielnik zegara dla ukladu generacji dead-time i filtra
 	TIM_TimeBaseStructure.TIM_RepetitionCounter=0;          		//licznik powtorzen
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;	//tryb pracy licznika
	TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);            	//Inicjalizacja licznika
	//Konfiguracja kanalu 4: generuje sygnal Trig
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;         	//tryb pracy kanalu
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;//wlaczenie generowania sygnalu na wyjsciu licznika
	TIM_OCInitStructure.TIM_Pulse = 10;                         // 10 us (10 mikrosekund)
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;   //polaryzacja wyjscia
	TIM_OC4Init(TIM1, &TIM_OCInitStructure);                    //Inicjalizacja kanalu 1 licznika TIM1	
	TIM_OC4PreloadConfig(TIM1, TIM_OCPreload_Enable);           //zapis ustawien, wlaczenie buforowania
	//Konfiguracja kanalu 1
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;         		// numer kanalu
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;  // wejscia dolaczone do ...  TIM_ICSelection_DirectTI
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;       // Capture performed each time an edge is detected on the capture input.
	TIM_ICInitStructure.TIM_ICFilter = 0;												// This parameter can be a number between 0x0 and 0xF */
	TIM_ICInit(TIM1, &TIM_ICInitStructure);
	//Konfiguracja kanalu 2
	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;         		// numer kanalu
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Falling;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;  // wejscia dolaczone do ...  TIM_ICSelection_DirectTI
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;       // Capture performed each time an edge is detected on the capture input.
	TIM_ICInitStructure.TIM_ICFilter = 0;												// This parameter can be a number between 0x0 and 0xF */
	TIM_ICInit(TIM1, &TIM_ICInitStructure);

	TIM_BDTRStructInit(&TIM_BDTRInitStruct);	
	TIM_BDTRConfig(TIM1, &TIM_BDTRInitStruct);
	TIM_CtrlPWMOutputs(TIM1, ENABLE);
	
	TIM_Cmd(TIM1, ENABLE);																			// Wlaczenie timera TIM1
	TIM_ITConfig(TIM1, TIM_IT_CC1, ENABLE);  // wlaczenie przerwania od CC1 dla TIM1
	TIM_ITConfig(TIM1, TIM_IT_CC2, ENABLE);  // wlaczenie przerwania od CC2 dla TIM1
}
	
/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/



