/**************************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 				  *
 *  ZL27ARM: PROJEKT_DMA  transfer pamiec <--> pamiec					*
 **************************************************************/
#include "stm32f10x.h"
#include <stdbool.h> 		// true, false
#include <stdio.h>
#include ".\..\Drivers\LCD1602\lcd_1602.h"

int  RCC_Config(void);
void GPIO_Config(void);
void NVIC_Config(void);
int SysTick_Config_Div8(unsigned int Ticks);

// zmienne globalne, SM = Stepper Motor -----------------------
unsigned char txt2LCD[32] = {0}; 		// bufor funkcji sprintf
const uint8_t txt1[]= "-pierwszy tekst-/0";
const uint8_t txt2[]= "- drugi  tekst -/0";

int main(void) {
	
  //------------------ konfiguracja systemu -------------------
  RCC_Config();   
  GPIO_Config(); 
  NVIC_Config(); 
  GPIO_ResetBits(GPIOB, 0xFF00u);// zgasimy wszystkie diody LED
	LCD_Initialize();              //inicjalizacja wyswietlacza
	LCD_SendCommand(LCD_DISPLAY_ONOFF | LCD_DISPLAY_ON | LCD_CURSOR_ON);
	SysTick_Config_Div8(9000000ul);

	//------------------------ program C ------------------------
	LCD_GoTo(0,1);
	LCD_WriteText((unsigned char *)"transfer DMA M2M\0");
	while (1) {
		//konfigurowanie ukladu DMA1, kanal 1
		DMA_InitTypeDef DMA_InitStructure;
		DMA_DeInit(DMA1_Channel1);																						// usun poprzednia konfiguracje DMA
		DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) txt2LCD;  			// adres docelowy transferu
		DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) txt1;         			// adres poczatku bloku do przeslania
		DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;                    // kierunek transferu
		DMA_InitStructure.DMA_BufferSize = 17;                                // liczba elementow do przeslania (dlugosc bufora)
		DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Enable;       // wlaczenie automatycznego zwiekszania adresu po stronie docelowej
		DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;               // wlaczenie automatycznego zwiekszania adresu po stronie pamieci
		DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;	// rozmiar pojedynczych przesylanych danych po stronie docelowej
		DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;       // rozmiar pojedynczych przesylanych danych po stronie pamieci
		DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;                       	// tryb dzialana kontrolera DMA - jeden transfer
		DMA_InitStructure.DMA_Priority = DMA_Priority_High;                   // priorytet DMA - wysoki
		DMA_InitStructure.DMA_M2M = DMA_M2M_Enable;                           // wlaczenie obslugi transferu z pamieci do pamieci
		DMA_Init(DMA1_Channel1, &DMA_InitStructure);                          // zapis konfiguracji
		DMA_Cmd(DMA1_Channel1, ENABLE);  // wlacz DMA, kanal 1
		
		LCD_GoTo(0,0);
		LCD_WriteText(txt2LCD);
		delay_us(8000000);
		
		//konfigurowanie ukladu DMA1, kanal 1
		DMA_DeInit(DMA1_Channel1);																						// usun poprzednia konfiguracje DMA
		DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) txt2LCD;  			// adres docelowy transferu
		DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) txt2;         			// adres poczatku bloku do przeslania
		DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;                    // kierunek transferu
		DMA_InitStructure.DMA_BufferSize = 17;                                // liczba elementow do przeslania (dlugosc bufora)
		DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Enable;       // wlaczenie automatycznego zwiekszania adresu po stronie docelowej
		DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;               // wlaczenie automatycznego zwiekszania adresu po stronie pamieci
		DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;	// rozmiar pojedynczych przesylanych danych po stronie docelowej
		DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;       // rozmiar pojedynczych przesylanych danych po stronie pamieci
		DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;                       	// tryb dzialana kontrolera DMA - jeden transfer
		DMA_InitStructure.DMA_Priority = DMA_Priority_High;                   // priorytet DMA - wysoki
		DMA_InitStructure.DMA_M2M = DMA_M2M_Enable;                           // wlaczenie obslugi transferu z pamieci do pamieci
		DMA_Init(DMA1_Channel1, &DMA_InitStructure);                          // zapis konfiguracji
		DMA_Cmd(DMA1_Channel1, ENABLE);  // wlacz DMA, kanal 1	
		
		LCD_GoTo(0,0);
		LCD_WriteText(txt2LCD);
		delay_us(8000000);
	}
}

//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
		// konfiguracja sygnalow taktujacych uzywanych peryferii
		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);   //wlacz taktowanie DMA		
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);// wlacz taktowanie portu GPIO B ( LEDy  )
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);// wlacz taktowanie portu GPIO C (LCD1602)

		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 

  // LED1 ... LED8 na plytce
  GPIO_InitStructure.GPIO_Pin = 0xFF00u; 	// piny 15,14,...9,8
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOB, &GPIO_InitStructure);
}

// ----------- konfiguracja kontrolera przerwan NVIC ----------
void NVIC_Config(void) {
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
}
// --------- konfiguracja licznika systemowego SysTick --------
int SysTick_Config_Div8(unsigned int Ticks){ 
  unsigned int ctrl;

  if (Ticks > SysTick_LOAD_RELOAD_Msk)  return 1;             	// czy wartosc poczatkowa nie przekracza max
                                                                
  SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        // ustaw wartosc poczatkowa licznika 
  NVIC_SetPriority (SysTick_IRQn, 0);                           // ustaw priorytet przerwania
  SysTick->VAL  = 0;                                            // ustaw wartosc aktualna licznika
  ctrl=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;      // ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	ctrl &= SysTick_CLKSource_HCLK_Div8;
  SysTick->CTRL = ctrl;                  // zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)
  return 0;
}
/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/



